/*
 * Copyright (C) oStorybook Team
 *
 * This program is free software, see <http://www.gnu.org/licenses/>.
 *
 * Parts of this code are from the NovaWorx project. Other parts are
 * from the SHEF project developed and published by Bob Tantlinger.
 */
package shef.editors.wys;

import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.IOException;
import javax.swing.Action;
import javax.swing.JEditorPane;
import javax.swing.KeyStroke;
import javax.swing.text.BadLocationException;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;
import shef.actions.CompoundUndoManager;
import i18n.I18N;
import resources.icons.ICONS;

public class HTMLEditPasteAction extends HTMLTextEditAction {

	public HTMLEditPasteAction(WysiwygEditor editor) {
		super(editor, I18N.getMsg("shef.paste"));
		putValue(MNEMONIC_KEY, I18N.getMnem("shef.paste"));
		setSmallIcon(ICONS.K.EDIT_PASTE);
		setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, InputEvent.CTRL_DOWN_MASK));
		addShouldBeEnabledDelegate((Action a) -> true);
		setShortDescription(I18N.getMsg("shef.paste"));
	}

	@Override
	protected void updateWysiwygContextState(JEditorPane wysEditor) {
		this.updateEnabledState();
	}

	@Override
	protected void wysiwygEditPerformed(ActionEvent e, JEditorPane editor) {
		HTMLEditorKit ekit = (HTMLEditorKit) editor.getEditorKit();
		HTMLDocument document = (HTMLDocument) editor.getDocument();
		Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
		try {
			CompoundUndoManager.beginCompoundEdit(document);
			Transferable content = clip.getContents(this);
			String txt = content.getTransferData(
					new DataFlavor(String.class, "String")).toString();

			document.replace(editor.getSelectionStart(),
					editor.getSelectionEnd() - editor.getSelectionStart(),
					txt, ekit.getInputAttributes());
		} catch (UnsupportedFlavorException | IOException | BadLocationException ex) {
		} finally {
			CompoundUndoManager.endCompoundEdit(document);
		}
	}

}
