package storybook.edit;

import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JCheckBox;
import org.miginfocom.swing.MigLayout;
import storybook.model.EntityUtil;
import storybook.model.book.Book;
import storybook.model.book.Book.TYPE;
import storybook.model.hbn.entity.AbstractEntity;
import storybook.model.hbn.entity.Strand;
import storybook.toolkit.swing.js.JSLabel;
import storybook.ui.MIG;
import storybook.ui.MainFrame;
import storybook.ui.interfaces.IRefreshable;
import storybook.ui.panel.AbstractPanel;

/**
 * Panel containing a list of check box
 *
 * @author favdb
 */
@SuppressWarnings("serial")
public class CheckBoxPanel extends AbstractPanel implements IRefreshable {

	private static final String TT = "CheckBoxPanel";

	public List<EntityCB> entityCB;
	private AbstractEntity entity, pointedEntity;
	public List<AbstractEntity> entities;
	public boolean autoSelect = true;
	private Book.TYPE search = Book.TYPE.NONE;
	private AbstractEntity toOmmit = null;

	public CheckBoxPanel(MainFrame mainFrame) {
		super(mainFrame);
	}

	public CheckBoxPanel(MainFrame mainFrame, Book.TYPE search) {
		super(mainFrame);
		this.search = search;
		initAll();
	}

	public CheckBoxPanel(MainFrame mainFrame, Book.TYPE search, List<AbstractEntity> toSel) {
		super(mainFrame);
		this.search = search;
		initAll();
		if (toSel != null) {
			selectEntities(toSel);
		}
	}

	@Override
	public void modelPropertyChange(PropertyChangeEvent evt) {
		// empty
	}

	@Override
	public void init() {
		//App.trace(TT+".init() for search="+search);
		entityCB = new ArrayList<>();
		reload();
	}

	public void reload() {
		//App.trace(TT+".reload()");
		entityCB.clear();
		@SuppressWarnings("unchecked")
		List<AbstractEntity> ls = EntityUtil.findEntities(mainFrame, search.toString());
		if (ls == null) {
			return;
		}
		ls.sort((o1, o2) -> o1.toString().compareTo(o2.toString()));
		ls.forEach(obj -> {
			if (!obj.equals(toOmmit)) {
				addEntity((AbstractEntity) obj);
			}
		});
	}

	public void addEntity(AbstractEntity entity) {
		//App.trace(TT+".addEntity(entity="+App.traceEntity(entity)+")");
		JCheckBox ck = new JCheckBox();
		ck.setOpaque(false);
		ck.addActionListener((ActionEvent evt) -> {
			pointedEntity = entity;
		});
		entityCB.add(new EntityCB(entity, ck));
		ck.setText(entity.toString());
		ck.setToolTipText(EntityUtil.getToolTip(entity));
		add(ck);
	}

	public void removeEntity(AbstractEntity entity) {
		int z = -1;
		if (entity == null) {
			return;
		}
		for (int i = 0; i < entityCB.size(); i++) {
			if (entity.equals(entityCB.get(i).entity)) {
				z = i;
			}
		}
		if (z != -1) {
			entityCB.remove(z);
			refresh();
			toOmmit = entity;
		}
	}

	@Override
	public void initUi() {
		//App.trace(TT+".initUi()");
		setLayout(new MigLayout(MIG.get(MIG.FILLX, MIG.WRAP, "ins 1")));
		//setBackground(Color.white);
		refresh();
	}

	@Override
	public void refresh() {
		//App.trace(TT+".refresh()");
		removeAll();
		entityCB.forEach(ecb -> {
			add(ecb.cb, MIG.SPLIT2);
			JSLabel js = new JSLabel(ecb.entity.getIcon());
			if (ecb.entity.getObjType() == TYPE.STRAND) {
				js.setBackground(((Strand) ecb.entity).getJColor());
			}
			add(js);
		});
		revalidate();
		repaint();
	}

	public void selectEntity(AbstractEntity ent) {
		JCheckBox cb = findCB(ent);
		if (cb != null) {
			cb.setSelected(true);
		}
	}

	private JCheckBox findCB(AbstractEntity ent) {
		for (EntityCB ecb : entityCB) {
			if (ecb.entity.getId().equals(ent.getId())) {
				return ecb.cb;
			}
		}
		return (null);
	}

	public void selectEntities(List<AbstractEntity> entities) {
		entities.forEach((e) -> selectEntity(e));
	}

	public List<AbstractEntity> getSelectedEntities() {
		ArrayList<AbstractEntity> ret = new ArrayList<>();
		entityCB.forEach((ecb) -> {
			if (ecb.cb.isSelected()) {
				ret.add(ecb.entity);
			}
		});
		return ret;
	}

	public AbstractEntity getPointedEntity() {
		return pointedEntity;
	}

	public AbstractEntity getEntity() {
		return entity;
	}

	public void setEntity(AbstractEntity entity) {
		this.entity = entity;
	}

	public void setEntityList(List<AbstractEntity> entities) {
		this.entities = entities;
	}

	public Book.TYPE getSearch() {
		return search;
	}

	public void setSearch(Book.TYPE search) {
		this.search = search;
	}

	public boolean getAutoSelect() {
		return autoSelect;
	}

	public void setAutoSelect(boolean flag) {
		this.autoSelect = flag;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		// empty
	}

	public static class EntityCB {

		AbstractEntity entity;
		JCheckBox cb;

		public EntityCB(AbstractEntity entity, JCheckBox cb) {
			this.entity = entity;
			this.cb = cb;
		}

		public JCheckBox getCB() {
			return cb;
		}
	}

}
